"use strict";

/**
 * @name Cart
 *
 * @description
 * Implements actions with cart
 *
 * @license http://www.wtfpl.net/ Do What The Fuck You Want To Public License
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
export default class cart {
	/**
	 * @name Type of the program
	 */
	static type = "module";

	/**
	 * Toggle
	 *
	 * Toggle the product in the cart (interface)
	 *
	 * @param {HTMLButtonElement|HTMLInputElement} element Handler elememnt of the product
	 * @param {HTMLElement} product The product element
	 * @param {bool} remove Remove the product element if json.amount === 0?
	 * @param {bool} force Ignore the damper? (false)
	 *
	 * @return {bool} Execution completed with an error?
	 */
	static toggle(element, product, remove = false, force = false) {
		// Blocking the element
		element.setAttribute("disabled", "true");

		core.modules.connect("damper").then(
			() => {
				// Imported the damper module

				// Execute under damper
				this.product.damper(
					element,
					product,
					"toggle",
					undefined,
					remove,
					force,
				);
			},
			() => {
				// Not imported the damper module

				// Execute
				this.product.system(
					element,
					product,
					"toggle",
					undefined,
					remove,
					force,
				);
			},
		);

		// Exit (success)
		return false;
	}

	/**
	 * Write
	 *
	 * Write the product in the cart (interface)
	 *
	 * @param {HTMLButtonElement|HTMLInputElement} element Handler elememnt of the product
	 * @param {HTMLElement} product The product element
	 * @param {number} amount Amount of writings
	 * @param {bool} remove Remove the product element if json.amount === 0?
	 * @param {bool} force Ignore the damper? (false)
	 *
	 * @return {bool} Execution completed with an error?
	 */
	static write(
		element,
		product,
		amount = 1,
		remove = false,
		force = false,
	) {
		// Blocking the element
		element.setAttribute("disabled", "true");

		core.modules.connect("damper").then(
			() => {
				// Imported the damper module

				// Execute under damper
				this.product.damper(
					element,
					product,
					"write",
					amount,
					remove,
					force,
				);
			},
			() => {
				// Not imported the damper module

				// Execute
				this.product.system(
					element,
					product,
					"write",
					amount,
					remove,
					force,
				);
			},
		);

		// Exit (success)
		return false;
	}

	/**
	 * Delete
	 *
	 * Delete the product from the cart (interface)
	 *
	 * @param {HTMLButtonElement|HTMLInputElement} element Handler elememnt of the product
	 * @param {HTMLElement} product The product element
	 * @param {number} amount Amount of deletings
	 * @param {bool} remove Remove the product element if json.amount === 0?
	 * @param {bool} force Ignore the damper? (false)
	 *
	 * @return {bool} Execution completed with an error?
	 */
	static delete(
		element,
		product,
		amount = 1,
		remove = false,
		force = false,
	) {
		// Blocking the element
		element.setAttribute("disabled", "true");

		core.modules.connect("damper").then(
			() => {
				// Imported the damper module

				// Execute under damper
				this.product.damper(
					element,
					product,
					"delete",
					amount,
					remove,
					force,
				);
			},
			() => {
				// Not imported the damper module

				// Execute
				this.product.system(
					element,
					product,
					"delete",
					amount,
					remove,
					force,
				);
			},
		);

		// Exit (success)
		return false;
	}

	/**
	 * Set
	 *
	 * Set amount of the product in the cart (interface)
	 *
	 * @param {HTMLButtonElement|HTMLInputElement} element Handler elememnt of the product
	 * @param {HTMLElement} product The product element
	 * @param {number} amount Amount of the product in the cart to be setted
	 * @param {bool} remove Remove the product element if json.amount === 0?
	 * @param {bool} force Ignore the damper? (false)
	 *
	 * @return {bool} Execution completed with an error?
	 */
	static set(
		element,
		product,
		amount = 1,
		remove = false,
		force = false,
	) {
		// Blocking the element
		element.setAttribute("disabled", "true");

		core.modules.connect("damper").then(
			() => {
				// Imported the damper module

				// Execute under damper
				this.product.damper(
					element,
					product,
					"set",
					amount,
					remove,
					force,
				);
			},
			() => {
				// Not imported the damper module

				// Execute
				this.product.system(
					element,
					product,
					"set",
					amount,
					remove,
					force,
				);
			},
		);

		// Exit (success)
		return false;
	}

	/**
	 * The product
	 *
	 * Handle the product in the cart (system)
	 *
	 * @param {HTMLButtonElement|HTMLInputElement} element Handler element of the product
	 * @param {HTMLElement} product The product element
	 * @param {string} type Type of action with the product
	 * @param {number} amount Amount of product to handle
	 * @param {bool} remove Remove the product element if json.amount === 0?
	 *
	 * @return {bool} Execution completed with an error?
	 */
	static product(
		element,
		product,
		type,
		amount = null,
		remove = false,
	) {
		core.modules.connect("damper").then(
			() => {
				// Imported the damper module

				// Execute under damper
				this.product.damper(
					element,
					product,
					type,
					amount,
					remove,
					force,
				);
			},
			() => {
				// Not imported the damper module

				// Execute
				this.product.system(
					element,
					product,
					type,
					amount,
					remove,
					force,
				);
			},
		);

		// Exit (success)
		return false;
	}

	/**
	 * @name Summary
	 *
	 * @description
	 * Initialize summary of products the cart (system)
	 *
	 * @return {Promise}
	 */
	static async summary() {
		// Request
		return await core.request("/cart/summary")
			.then((json) => {
				if (json) {
					// Received a JSON-response

					if (
						json.errors !== null &&
						typeof json.errors === "object" &&
						json.errors.length > 0
					) {
						// Fail (received errors)
					} else {
						// Success (not received errors)

						// Initializing the summary amount <span> element
						const amount = document.getElementById("amount");

						if (amount instanceof HTMLElement) {
							// Initialized the summary amount element

							// Writing summmary amount into the summary amount element
							amount.innerText = json.amount;
						}

						// Initializing the summary cost <span> element
						const cost = document.getElementById("cost");

						if (cost instanceof HTMLElement) {
							// Initialized the summary cost element

							// Writing summmary cost into the summary cost element
							cost.innerText = json.cost;
						}
					}
				}
			});
	}
}

core.modules.connect("damper").then(() => {
	// Imported the damper module

	Object.assign(
		cart.product,
		{
			/**
			 * Toggle
			 *
			 * Toggle the product in the cart (damper)
			 *
			 * @param {HTMLButtonElement|HTMLInputElement} element Handler elememnt of the product
			 * @param {HTMLElement} product The product element
			 * @param {bool} remove Remove the product element if json.amount === 0?
			 * @param {bool} force Ignore the damper? (false)
			 *
			 * @return {Promise}
			 */
			damper: core.damper(
				(...variables) => cart.product.system(...variables).then(cart.summary),
				300,
				6,
			),
		},
	);
});

Object.assign(
	cart.product,
	{
		/**
		 * The product
		 *
		 * Handle the product in the cart (system)
		 *
		 * @param {HTMLButtonElement|HTMLInputElement} element Handler element of the product
		 * @param {HTMLElement} product The product element
		 * @param {string} type Type of action with the product
		 * @param {number} amount Amount of product to handle
		 * @param {bool} remove Remove the product element if json.amount === 0?
		 *
		 * @return {Promise|null}
		 */
		async system(
			element,
			product,
			type,
			amount = null,
			remove = false,
			resolve = () => {},
			reject = () => {},
		) {
			try {
				if (
					(element instanceof HTMLButtonElement ||
						element instanceof HTMLInputElement) &&
					product instanceof HTMLElement
				) {
					// Validated

					// Initializing the buffer of request body
					let request = "";

					// Initializing of identifier of the product
					const identifier = +product.getAttribute(
						"data-product-identifier",
					);

					if (typeof identifier === "number") {
						// Validated identifier

						// Writing to the buffer of request body
						request += "&identifier=" + identifier;

						if (
							type === "toggle" ||
							type === "write" ||
							type === "delete" ||
							type === "set"
						) {
							// Validated type

							// Writing to the buffer of request body
							request += "&type=" + type;

							console.log(type, amount);
							if (
								(type === "toggle" &&
										amount === null ||
									typeof amount === "undefined") ||
								(type === "set" &&
										amount === 0 ||
									amount === 100) ||
								typeof amount === "number" &&
									amount > 0 &&
									amount < 100
							) {
								// Validated amount

								console.log(amount);
								if (type !== "toggle") {
									// Not a toggle request

									// Writing to the buffer of request body
									request += "&amount=" + amount;
								}

								// Request
								return await core.request(
									"/cart/product",
									request,
								)
									.then(
										(json) => {
											if (json) {
												// Received a JSON-response

												if (
													json.errors !== null &&
													typeof json.errors === "object" &&
													json.errors.length > 0
												) {
													// Fail (received errors)
												} else {
													// Success (not received errors)

													if (remove && json.amount === 0) {
														// Requested deleting of the product element when there is no the product in the cart

														// Deleting the product element
														product.remove();
													} else {
														// Not requested deleting the product element when there is no the product in the cart

														// Unblocking the element
														element.removeAttribute("disabled");

														// Writing offset of hue-rotate to indicate that the product is in the cart
														product.style.setProperty(
															"--hue-rotate-offset",
															json.amount + "0deg",
														);

														// Writing attribute with amount of the product in the cart
														product.setAttribute(
															"data-product-amount",
															json.amount,
														);

														// Initializing the amount <span> element
														const amounts = product.querySelectorAll(
															'[data-product-parameter="amount"]',
														);

														for (const amount of amounts) {
															// Iterating over an amount elements

															if (amount instanceof HTMLInputElement) {
																// The <input> element

																// Writing amount of the product in the cart
																amount.value = json.amount;
															} else {
																// Not the <input> element

																// Writing amount of the product in the cart
																amount.innerText = json.amount;
															}
														}
													}

													// Exit (success)
													resolve();
												}
											}
										},
										() => reject(),
									);
							}
						}
					}
				}
			} catch (e) {
				// Exit (fail)
				reject(e);
			}
		},
	},
);

// Connecting to the core
if (!core.cart) core.cart = cart;
